function a = satlin(n,b)
%SATLIN   Saturated Linear Activation Transfer Function.
%         Returns 0 for elements of N below the bias,
%         1 for elements above 1+bias. Elements between
%         bias and 1+bias are returned unchanged.
%         (See PURELIN)
% 
%         SATLIN(N)
%           N - SxQ matrix of net input vectors.
%         Returns the linearly saturated elements of N.
%
%         SATLIN(N,B) ...Used when Batching.
%           B - Sx1 vector of biases.
%         Returns the N using the bias in each for of B
%           for the elements in corresponding rows of N.

%         M.H. Beale & H.B. Demuth, 1-31-92
%         Copyright (c) 1992 by the MathWorks, Inc.

if nargin < 1 | nargin > 2
  error('Wrong number of arguments.');
  end

if nargin==1
  a = (n >= -1 & n <= 1).*n + (n > 1) - (n < -1);
else
  [nr,nc] = size(n);
  x = n+b*ones(1,nc);
  a = (x >= -1 & x <= 1).*x + (x > 1) - (x < -1);
  end
