function w=koh_2(matr,Tmax,Emin,dHmax,Kmax,dHmin,Kmin,Vig,ETA,KT,KV);

%	Functia KOH_2 antreneaza o retea Kohonen destinata clasificarii
%	unor grafice de sarcina pe categorii de consum.
%	Antrenarea se face dupa principiul "cresterii retelei", dupa modelul:
%	se porneste cu un singur neuron, si se adauga progresiv altii neuroni
%	de fiecare data cind distanta minima dintre un model si neuronii
%	existenti depaseste o valoare prag, numita "vigilenta"

%	Date de intrare :

%	G - matricea [NP+1,Pat] a graficelor de sarcina :
%		G(1:NP,k) - graficul de sarcina k in unitati absolute (k=1..Pat);
%		G(NP+1,k) - tipul consumatorului asociat sarcinii k, atunci cind
%			    acesta este cunoscut (se aplica in cazul graficelor
%			    generate pornind de la un numar de grafice tip);
%	Tmax - numarul maxim de iteratii;
%	Emin - abaterea minima intre norma matriceala a ponderilor retelei in doua
%	       oteratii succesive (folosita pentru oprirea procesului iterativ);
%	dHmax, dHmin - fereastra de timp de o parte si alta a palierului la care
%		       apare sarcina maxima, respectiv minima, folosite pentru
%		       ponderea distantelor intre grafic si ponderile existente;
%	Kmax, Kmin -   coeficienti de ponderare a distantelor intre graficul de
%		       sarcina si ponderile existente pentru virfiul, respectiv
%		       golul de sarcina;
%	Vig - vigilenta;
%	ETA - rata de invatare;
%	KT - coeficient de atenuare a ratei de invatare;
%	KV - coeficient de atenuare a vigilentei;

%	Date de iesire :

%	w - matricea [NP,NK] ponderilor in reteaua Kohonen dupa antrenare;

%	Alte date :

%	Pat - numarul de modele din setul de antrenare;
%	NP - numarul de paliere ale graficelor de sarcina;
%	NK - numarul de neuroni din reteaua Kohonen la un moment dat sau dupa antrenare;
%	eps - nivelul zgomotului care se introduce peste un model la asocierea
%	      acestuia la un neuron care se adauga in retea (1-eps , 1+eps);


eps=0.0;

%	load c:\matfiles\date_10k.mat;

G=matr;

[NP,Pat]=size(G);
NP=NP-1;


Tip=G(NP+1,:);
G=G(1:NP,:);

	% Normalizarea modelelor (graficelor de sarcina)

for k=1:Pat
	%G(:,k)=G(:,k)/norm(G(:,k));
    G(:,k)=G(:,k)/sum(G(:,k))*100;
end;

	% Initializari pentru intrarea in bucla iterativa

Err=0.5;		% Abaterea-norma intre matricele ponderilor in doua iteratii succesive
NK=1;		% Numarul de neuroni din reteaua Kohonen
w=G(:,1);	% Ponderile asociate primului neuron

	% Adaugarea unui zgomot peste ponderile initiale

x=1+eps*(2*rand(NP,1)-1);
w=w.*x;

t=1;

while (t<=Tmax) & (Err>Emin)

		% Memorarea ponderilor si numarului de neuroni la
		% inceputul iteratiei t

	OldW=w;
	OldNK=NK;

		% Prezentarea fiecarui model M retelei

	for M=1:Pat

			% Pregatirea vectorului K de ponderare a distantelor
			% dintre modelul M si neuronii din retea

		K=ones(NP,1);

			% Virf de sarcina

		GMax=max(G(:,M));
		K=h_minmax(K,G(:,M),GMax,dHmax,Kmax);

			% Gol de sarcina

		GMin=min(G(:,M));
		K=h_minmax(K,G(:,M),GMin,dHmin,Kmin);

			% Distantele intre modelul M si cei NK neuroni

		for n=1:NK
			d1=(G(:,M)-w(:,n)).*(G(:,M)-w(:,n));
			d(n)=K'*d1;
		end;

			% Distanta minima

		dM=min(d);

			% Test vigilenta
%dM
%pause
		if dM<=Vig

				% Nu se creaza noi neuroni
				% Se stabileste neuronul cistigator, caruia
				% i se adapteaza ponderile

			Win=find(d==dM);
			w(:,Win)=w(:,Win)+ETA*(G(:,M)-w(:,Win));
%			w(:,Win)=w(:,Win)/norm(w(:,Win));
            w(:,Win)=w(:,Win)/sum(w(:,Win))*100;
%	WinN(t,M)=Win;
		   else

				% Se creaza un nou neuron ale carui ponderi
				% se initializeaza cu modelul curent,
				% peste care se suprapune un zgomot

			NK=NK+1;
			w(:,NK)=G(:,M);
			x=1+eps*(2*rand(NP,1)-1);
			w(:,NK)=w(:,NK).*x;
%	WinN(t,M)=NK;
		end;
	end;		% for M=1:Pat

		% Adaptare ratei de invatare si a vigilentei

	ETA=ETA*(1-exp(-t/KT));
	Vig=Vig*(1-exp(-t/KV));

		% Daca nu s-au adaugat neuroni se determina abaterea-norma
		% intre ponderilede la inceputul si finalul interatiei

	if OldNK==NK
		Err=norm(w-OldW);
	end;
Er(t)=Err+NK;

		% Trecerea la o noua iteratie

	t=t+1;
end;

Er

%	WinN