function [popx,popy,fdmax,fd,rezultate]=alggen(N,rincr,rmut,maxgen)

% Determinarea amplasarii optime a unui punct de conexiune radiala
% intr-o retea data de un numar de puncte definite tabelar, prin perechi (x,y)
% folosind un algoritm genetic
% Coordonatele punctului cautat se codifica binar (0001001110101),
% in stringuri separate pentru coordonata x si y, care constituie
% populatia AG. 
%
% Apelul functiei:
%     [x,fd,f]=alggen(X,N,rincr,rmut,maxgen);
%
% Date de intrare:
%   X - vector de forma (x1,y1,x2,y2,... xk,yk,...)continand perechile (x,y) ale punctelor;
%   N - dimensiunea populatiei AG
%   rincr - rata de incrucisare (0.6 - 0.97)
%   rmut - rata mutatiilor  0.001 - 0.1
%   maxgen - numarul de generatii (iteratii)
%
% Date de iesire:
%   popx, popy  - matricea populatiei algoritmului genetic;
%   fdmax - functia de adaptare pentru c.m. bun cromozom din fiecare iteratie (inversul distantelor);
%   fd  - functia de adaptare a cromozomilor din ultima iteratie

% conversia valorilor din binar in numeric se va face cu functia bin2dec bin2dec('010111')

param=10000;
bits=14;

X=[2 8 5 2 8 8 5 9 8 3 2 3].*(param/10);
lung=max(size(X));
coordx=[];coordy=[];

% separarea coordonatelor x si y in 2 vectori distincti 
for i=1:2:lung-1
	coordx=[coordx X(i)];
end;
for j=2:2:lung
	coordy=[coordy X(j)];
end;


% generarea populatiei initiale pentru algoritm
% pentru coordonata x si y
[popx,poprealx]=populatie(N,bits,param);
[popy,poprealy]=populatie(N,bits,param);

popx(1,:)=dec2bin(5000,bits);
popy(1,:)=dec2bin(5500,bits);



% mesaj='populatia initiala'
% x
%pause;
gen=0;
fdmax=[];
lungmin=[];
% pause;
optim1=[];
rezultate=[];
while gen<maxgen
    gen=gen+1
    
 % calculul functiei de adaptare (inversul distantei parcurse)
  if gen==1
 [fd,f,dist]=fevala(popx, popy, coordx, coordy);
  end;
% mesaj='functia de adaptare pentru populatia initiala'
% d
% fd
% f
%pause;
% mesaj='selectie'
% selectie formarea populatiei pentru incrucisare xp
xp=[];yp=[];
fad1=[];
kp=[];
for k=1:N
	jp=selectie(fd,f,N);
    kp=[kp jp];
	czx=popx(jp,:);czy=popy(jp,:);
    cfd=fd(1,jp);
	xp=[xp;czx];yp=[yp;czy];
    fad1=[fad1 cfd];
end;

xp1=xp;
xp;
% mesaj='functia de adaptare dupa selectie'
fd=fad1;

%pause;
% incrucisare
% mesaj='incrucisare'
np=N;
for k=1:np/2
	% selectarea parintilor
	ii=ceil(rand*np);
	jj=ii;
    while jj==ii
    jj=ceil(rand*np);
    end;
	% formarea urmasilor
	p=rand;
	px1=xp(ii,:);py1=yp(ii,:);
	px2=xp(jj,:);py2=yp(jj,:);
	if p<rincr
		[ux1,ux2]=incr(px1,px2);
        [uy1,uy2]=incr(py1,py2);
    else 
		ux1=xp(ii,:);uy1=yp(ii,:);
		ux2=xp(jj,:);uy2=yp(jj,:);
	end;
	% trecerea pe ultimele locuri a cromozomilor parinti
    tempx1=xp(np,:); tempx2=xp(np-1,:);  tempy1=yp(np,:); tempy2=yp(np-1,:);
	xp(np,:)=xp(ii,:); xp(np-1,:)=xp(jj,:); yp(np,:)=yp(ii,:); yp(np-1,:)=yp(jj,:);
	xp(ii,:)=tempx1; xp(jj,:)=tempx2; yp(ii,:)=tempy1; yp(jj,:)=tempy2;

	% inscrierea cromozomilor urmasi in populatia curenta
      xp(np,:)=ux1; yp(np,:)=uy1;
      xp(np-1,:)=ux2; yp(np-1,:)=uy2;
     
end; % for k

% mesaj='populatia dupa incrucisare'
% xp
%pause;
% mutatii
for k=1:N
	p=1;
	if p<rmut
       % mesaj='se face mutatie'
		 xmut=xp(k,:);
         g1=bits+1;
             while (g1>bits)
                 g1=ceil(rand*100);
              end; 
             if xmut(g1)=='0'
                 xmut(g1)='1';
             else
                 xmut(g1)='0';
             end;
    end;
    p=1;
    if p<rmut     
          ymut=yp(k,:);
          g1=bits+1;
             while (g1>bits)
                 g1=ceil(rand*100);
              end; 
             if ymut(g1)=='0'
                 ymut(g1)='1';
             else
                 ymut(g1)='0';
             end;
                 
             xp(k,:)=xmut;
             yp(k,:)=ymut;
             
    end;
end;

% mesaj='populatia dupa mutatii'
xp3=xp;
%pause;

% recalcularea functiei de adaptare pentru populatia modificata
 [fd,f,dist]=fevala(popx, popy, coordx, coordy);

% elitism
% memorarea celui mai adaptat cromozom din gen.curenta pentru elitism
maxfad=max(fd);
fdmax=[fdmax maxfad];
lungmin=[lungmin min(dist)];
lungtraseumin=min(dist);
for q=1:N
	if fd(q)==maxfad
		elitax=popx(q,:);elitay=popy(q,:);
        fdelita=fd(1,q);
        
	end;
end;

% se inlocuieste cromozomul cel mai slab adaptat cu elita
% mesaj='cromozom eliminat la elitism'
minfad=min(fd);
elim=1/minfad;
for r=1:N
	if fd(r)==minfad
		xp(r,:)=elitax;
        yp(r,:)=elitay;
         break;
	end;
end;
%elita


%mesaj='populatia dupa mutatii'
popx=xp;
popy=yp;
fdmax
elitax
elitay
bin2dec(elitax)
bin2dec(elitay)
lungtraseumin
pause;

rezultate=[rezultate;[gen bin2dec(elitax) bin2dec(elitay) lungtraseumin]];

end;