function [tabu,d,drmin,drummin,Lmin,tabopt]= antzrev(N,NA,c,ro,alpha,beta,a,tmax)

% functia calculeaza drumul minim intre o serie de N puncte
% date prin coordonatele lor (x1,y1, x2,y2,...,xN,yN), folosind
% algoritmul furnicii, precum si traseul cu pierderi minime
%
% sintaxa:
% [tabu,d, distopt, tabopt]= antz(N,NA,c,ro,alpha,beta,a)
%
% tabu - lista in care se inregistreaza ordinea parcurgerii
%        punctelor de catre fiecare agent
% d - matricea distantelor dintre puncte
% N - numarul de puncte
% NA - numarul de agenti (furnici)
% c - concentratia de feromoni initiala
% ro - factor de evaporare
% alpha - ponderea concentratiei de feromoni
% beta - ponderea vizibilitatii
% a - vectorul coordonatelor consumatorilor a=(x1,y1, x2,y2,...,xN,yN)
% rez - rezistenta specifica a liniei electrice
% cgrup - vectorul curentilor absorbiti de consumatorii din grup
% Q - cantitatea totala de feromoni a unui agent
% t - contor de iteratii
% tmax - numar maxim de iteratii
% distopt - drumul minim calculat de algoritm
% tabopt - bucla cu distanta minima parcursa de un agent
% distalg - lungimea totala a drumului optim


lung=max(size(a));
dist=[];
x=[];y=[];
traseuopt=[];
distparcurs=[];
% separarea coordonatelor x si y in 2 vectori distincti
for i=1:2:lung-1
    x=[x a(i)];
end;

for j=2:2:lung
    y=[y a(j)];
end;

x
y

Q=100;

% initializarea concentratiilor de feromoni de pe laturi si
% calculul distantelor dintre noduri
tau=zeros(N,N);
d=tau;

for i=1:N
    for j=1:N
        if i ~=j
            tau(i,j)=c;
            dtau(i,j)=0;
        end;
        if i<j
            d(i,j)=sqrt((x(i)-x(j))^2+(y(i)-y(j))^2);
        else if i>j
                d(i,j)=d(j,i);
            end;
        end;
    end; %for j
end; %for i

% amplasarea agentilor in noduri
m=NA/N;
for k=1:NA
    o=floor((k-1)/m);
    oras(k)=o+1;
    oras1(k)=oras(k);
end; %for k
Lmin=100000000;
t=1;

% ciclul iterativ
while t<=tmax
    t
    % initializarea listelor tabu
    tabu=zeros(NA,N+1);
    
    s=1;
    
    % inserarea primului oras in fiecare lista tabu
    for k=1:NA
        o=floor((k-1)/m);
        oras(k)=o+1;
        tabu(k,1)=oras(k);
    end;
    
    % completarea listelor tabu
    while s<N
        s=s+1;
        suma=zeros(1,NA);
        for k=1:NA     % pentru fiecare agent
            % amplasarea agentului in orasul de origine
            i=tabu(k,s-1);
            %  i=oras(k);
            % probabilitatile de deplasare in orasele vecine
            % se verifica daca orasul de destinatie exista in lista tabu a agentului k
            zz=tabu(k,:);
            suma(k)=0;
            for j=1:N  % pentru toate orasele
                verif=0;
                for v=1:N+1
                    if zz(v)==j
                        verif=1;
                    end; % if zz(v)
                end; %for v
                % daca orasul nu se gaseste in lista tabu, se adauga un
                % termen la suma de la numitor
                if verif==0
                    bb=tau(i,j)^alpha;
                    dd=(1/d(i,j))^beta;
                    suma(k)=suma(k)+bb*dd;
                end;  %if verif
            end;  %for j
            
            % calculul probabilitatilor de deplasare in orasele vecine
            for j=1:N
                verif=0;
                for v=1:N+1
                    if zz(v)==j
                        verif=1;
                    end; % if zz(v)
                end; %for v
                
                if verif==0
                    bb=tau(i,j)^alpha;
                    dd=(1/d(i,j))^beta;
                    P(k,j)=bb*dd/suma(k);
                else
                    P(k,j)=0;
                end % if verif
            end; % for j
        end; % for k
        %P
        %pause;
        % selectarea orasului de destinatie jj
        for k=1:NA  % penru fiecare agent
            jj=0;
            verif=1;
            PD=P(k,:);
            fsum=sum(PD);
            while verif==1    % orasul se inscrie in lista tabu doar daca nu exista deja acolo
                % alegerea orasului de destinatie dupa regula ruletei
                val=rand*fsum;
                contor=0; sm=0;
                while (sm<val) & (contor<=N)
                    contor=contor+1;
                    sm=sm+PD(contor);
                end;  % while sm
                zz=tabu(k,:);
                % se verifica daca orasul index exista deja in lista tabu
                verif=0;
                for v=1:N+1
                    if zz(v)==contor
                        verif=1;
                    end; % if zz(v)
                end; %for v
            end % while verif
            jj=contor;
            tabu(k,s)=contor;
        end; % for k
        
    end;  % while s<N
    
    
    for k=1:NA % pentru fiecare agent
        % mutarea agentilor in orasele de origine
        tabu(k,N+1)=tabu(k,1);
        % calculul lungimii totale parcurse
        L(k)=0;
        for p=1:N
            i=tabu(k,p);
            j=tabu(k,p+1);
            L(k)=L(k)+d(i,j);
        end;  % for p
        % actualizarea traseului de lungime minima
        if L(k)<Lmin
            Lmin=L(k);
            tabopt=tabu(k,:);
            
        end; % if L(k)
    end;  % for k
    
    
    % actualizarea concentratiilor de feromoni de pe laturi
    dtau=zeros(NA,N+1);
    for k=1:NA
        for p=1:N
            i=tabu(k,p);
            j=tabu(k, p+1);
            dtau(i,j)=dtau(i,j)+Q/L(k);
        end; % for p
    end; % for k
    
    for i=1:N
        for j=1:N
            if i~=j
                tau(i,j)=ro*tau(i,j)+dtau(i,j);
            end; % if
        end;
    end; %for
    
    % memorarea traseului minim curent
    traseuopt=[traseuopt;tabopt]
    distparcurs=[distparcurs;Lmin]
    pause;
    t=t+1;
end; % while t<tmax

%           verificarea distantei optime obtinute
% calculul distantei parcurse de fiecare agent, pe tronsoane
drmin=[];
for i=1:N
    drmin=[drmin d(tabopt(i),tabopt(i+1))];
end;
drmin=drmin';
drummin=sum(drmin);
